# Generated by Django 3.2.3 on 2021-05-29 22:14

import uuid

import django.db.models.deletion
from django.apps.registry import Apps
from django.conf import settings
from django.db import migrations, models
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.utils.timezone import now

import authentik.core.models


def migrate_sessions(apps: Apps, schema_editor: BaseDatabaseSchemaEditor):
    db_alias = schema_editor.connection.alias
    from django.contrib.sessions.backends.cache import KEY_PREFIX, SessionStore
    from django.core.cache import cache

    AuthenticatedSession = apps.get_model("authentik_core", "AuthenticatedSession")
    User = apps.get_model("authentik_core", "user")

    session_keys = cache.keys(KEY_PREFIX + "*")
    for key in session_keys:
        key = key.replace(KEY_PREFIX, "")
        store = SessionStore(key)
        data = store.load()
        if data == {} or "_auth_user_id" not in data:
            continue
        if (
            AuthenticatedSession.objects.using(db_alias)
            .filter(session_key=key)
            .exists()
        ):
            continue
        users = User.objects.using(db_alias).filter(pk=data.get("_auth_user_id"))
        if not users.exists():
            continue
        AuthenticatedSession.objects.using(db_alias).create(
            session_key=key,
            user=users.first(),
            expires=data.get("_session_expiry", now()),
        )


class Migration(migrations.Migration):

    dependencies = [
        ("authentik_core", "0021_alter_application_slug"),
    ]

    operations = [
        migrations.CreateModel(
            name="AuthenticatedSession",
            fields=[
                (
                    "expires",
                    models.DateTimeField(
                        default=authentik.core.models.default_token_duration
                    ),
                ),
                ("expiring", models.BooleanField(default=True)),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                ("session_key", models.CharField(max_length=40)),
                ("last_ip", models.TextField()),
                ("last_user_agent", models.TextField(blank=True)),
                ("last_used", models.DateTimeField(auto_now=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.RunPython(migrate_sessions),
    ]
